#include <iostream>
#include <cstdint>
#include <vector>
#include <cassert>

const uint32_t INF = 1e9;

struct Point {
	uint32_t x, y;
	Point(uint32_t x = 0, uint32_t y = 0): x(x), y(y) {}
	uint32_t colour() const {
		return (x + y) % 2;
	}
};

struct Vertex {
	Point p;

	uint32_t comp;
	std::vector<Vertex*> edges;

	uint32_t vis;
	Vertex *m;
	bool hasAdj;
	bool cover;

	Vertex(): comp(INF), vis(0), m(nullptr), hasAdj(false), cover(false) {}

	//called from "left" half
	bool kuhn(uint32_t curVis) {
		if (vis == curVis) {
			return false;
		}
		vis = curVis;
		for (Vertex *v : edges) {
			if (v->vis == vis) {
				continue;
			}
			v->vis = vis;
			if (v->m == nullptr) {
				v->m = this;
				m = v;
				return true;
			}

			if (v->m->kuhn(vis)) {
				v->m = this;
				m = v;
				return true;
			}
		}
		return false;
	}

	void mark_adj() {
		if (m) {
			return;
		}
		for (Vertex *v : edges) {
			v->hasAdj = true;
		}
	}

	void dfs_comp() {
		for (Vertex* c : edges) {
			if (c->comp != INF) {
				continue;
			}
			c->comp = comp;
			c->dfs_comp();
		}
	}
};

const uint32_t H_CNT = 8;
const uint32_t MAX_N = 105;

uint32_t dX[] = {1, 2, (uint32_t) -1, (uint32_t) -2, 1, 2, (uint32_t) -1, (uint32_t) -2};
uint32_t dY[] = {2, 1, 2, 1, (uint32_t) -2, (uint32_t) -1, (uint32_t) -2, (uint32_t) -1};

uint32_t w, h, maxJump;

bool empty[MAX_N][MAX_N];

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	std::cin >> h >> w >> maxJump;
	//assert(maxJump % 2 == 0);
	for (uint32_t y = 0;y < h;y++) {
		for (uint32_t x = 0;x < w;x++) {
			char c;
			std::cin >> c;
			empty[x][y] = c == '.';
		}
	}
}

Vertex cells[MAX_N][MAX_N];

struct Component {
	std::vector<Vertex*> left;
	std::vector<Vertex*> right;

	std::vector<Point> minCov[2];
	std::vector<Point> notCov[2];
};
std::vector<Component> comp;

void build_graph() {
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < h;y++) {
			if (!empty[x][y]) {
				continue;
			}

			for (uint32_t d = 0;d < H_CNT;d++) {
				uint32_t nX = x + dX[d];
				uint32_t nY = y + dY[d];

				if ((nX >= w) || (nY >= h)) {
					continue;
				}
				if (!empty[nX][nY]) {
					continue;
				}
				cells[x][y].edges.push_back(&cells[nX][nY]);
			}
		}
	}
}

void build_components() { 
	uint32_t cCnt = 0;
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < h;y++) {
			if ((!empty[x][y]) || (cells[x][y].comp < INF)) {
				continue;
			}
			cells[x][y].comp = cCnt++;
			cells[x][y].dfs_comp();
		}
	}

	comp.resize(cCnt);
	
	for (uint32_t x = 0;x < w;x++) {
		for (uint32_t y = 0;y < h;y++) {
			if (!empty[x][y]) {
				continue;
			}
			cells[x][y].p = Point(x, y);
			Component &c = comp[cells[x][y].comp];
			if (cells[x][y].p.colour()) {
				c.right.push_back(&cells[x][y]);
			} else {
				c.left.push_back(&cells[x][y]);
			}
		}
	}
}

void build_cover(Component &c) {
	uint32_t matchCnt = 0;
	for (uint32_t i = 0;i < c.left.size();i++) {
		if (c.left[i]->m == nullptr) {
			matchCnt += c.left[i]->kuhn(i + 1);
		}
	}
	//std::cerr << "matching " << matchCnt << "\n";
	for (Vertex *v : c.left) {
		v->mark_adj();
	}
	for (Vertex *v : c.right) {
		v->mark_adj();
	}
	
	for (Vertex *v : c.left) {
		if (v->m != nullptr) {
			if (v->hasAdj) {
				v->cover = true;
			} else {
				v->m->cover = true;
			}
		}
	}

	for (Vertex *v : c.left) {
		if (v->cover) {
			c.minCov[0].push_back(v->p);
		} else {
			c.notCov[0].push_back(v->p);
		}
	}
	for (Vertex *v : c.right) {
		if (v->cover) {
			c.minCov[1].push_back(v->p);
		} else {
			c.notCov[1].push_back(v->p);
		}
	}

//	std::cerr << "Min Cover: ";
//	for (Point p : c.minCov) {
//		std::cerr << "(" << p.x << " " << p.y << ") ";
//	}
//	std::cerr << "\nNot Cover: ";
//	for (Point p : c.notCov) {
//		std::cerr << "(" << p.x << " " << p.y << ") ";
//	}
//	std::cerr << "\n\n";
}

std::vector<Point> ans;



void build_ans(Component &c) {
	if (c.left.size() + c.right.size() == 1) {
		if (!c.left.empty()) {
			ans.push_back(c.left[0]->p);
		} else {
			ans.push_back(c.right[0]->p);
		}
		return;
	}

	build_cover(c);
	ans.insert(ans.end(), c.minCov[1].begin(), c.minCov[0].end());
	ans.insert(ans.end(), c.minCov[0].begin(), c.minCov[1].end());
	ans.insert(ans.end(), c.notCov[0].begin(), c.notCov[0].end());
	ans.insert(ans.end(), c.notCov[1].begin(), c.notCov[1].end());

	for (uint32_t i = 0;i < maxJump;i++) {
		if (i & 1) {
			ans.insert(ans.end(), c.notCov[0].begin(), c.notCov[0].end());
			ans.insert(ans.end(), c.notCov[1].begin(), c.notCov[1].end());
		} else {
			ans.insert(ans.end(), c.minCov[0].begin(), c.minCov[0].end());
			ans.insert(ans.end(), c.minCov[1].begin(), c.minCov[1].end());
		}
	}
}

void print_ans() {
	std::cout << ans.size() << "\n";
	for (const Point &p : ans) {
		std::cout << p.y << " " << p.x << "\n";
	}
}

int main() {
	input();
	build_graph();
	build_components();
	for (auto &i : comp) {
		build_ans(i);
	}
	print_ans();
}
